//------------------------------------------------------------------
//FLASH256K Controller AHB Interface top module
//------------------------------------------------------------------

module Gowin_Flash_Controller_Template
(
    input   wire        clk,
    input   wire        rstn,
    input   wire        key_write,
    input   wire        key_read,
    input   wire        key_erase,
    output  wire [31:0] data_out
);

//get the falling edge of the key_write
reg fall_key_write1,fall_key_write2;
wire fall_key_write = ~fall_key_write1  & fall_key_write2;

always @(posedge clk or negedge rstn)
begin
    if(~rstn)
    begin
        fall_key_write1 <= 1'b0;
        fall_key_write2 <= 1'b0;
    end
    else
    begin
        fall_key_write1 <= key_write;
        fall_key_write2 <= fall_key_write1;
    end
end

//get the falling edge of the key_read
reg fall_key_read1,fall_key_read2;
wire fall_key_read = ~fall_key_read1 & fall_key_read2;

always @(posedge clk or negedge rstn)
begin
    if(~rstn)
    begin
        fall_key_read1 <= 1'b0;
        fall_key_read2 <= 1'b0;
    end
    else
    begin
        fall_key_read1 <= key_read;
        fall_key_read2 <= fall_key_read1;
    end
end

reg fall_key_erase1,fall_key_erase2;
wire fall_key_erase = ~fall_key_erase1 & fall_key_erase2;

always @(posedge clk or negedge rstn)
begin
    if(~rstn)
    begin
        fall_key_erase1 <= 1'b0;
        fall_key_erase2 <= 1'b0;
    end
    else
    begin
        fall_key_erase1 <= key_erase;
        fall_key_erase2 <= fall_key_erase1;
    end
end

wire[31:0] AHB_HRDATA;
wire [1:0] AHB_HRESP;
wire [2:0] AHB_HSIZE;

reg [31:0] AHB_HWDATA;
reg [31:0] AHB_HADDR;
reg        AHB_HSEL;
reg        AHB_HWRITE;
reg [ 1:0] AHB_HTRANS; 
reg [ 7:0] i;

always @(posedge clk or negedge rstn)
begin
    if(~rstn)
    begin
        i               <= 8'd0;
        AHB_HSEL        <= 1'b0;
        AHB_HWDATA      <= 32'b0;
        AHB_HADDR       <= 32'b0;
        AHB_HWRITE      <= 1'b0;
        AHB_HTRANS      <= 2'b10;
    end
    else
    begin
        case (i)
            8'd0:
            begin
                if(fall_key_write)
                begin
                    i <= 8'd13;
                end 
                else if(fall_key_read)
                begin
                    i <= 8'd27;
                end
                else if(fall_key_erase)
                begin
                    i <= 8'd1;
                end
            end
            8'd1://write xaddress = 0
            begin
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b1;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd2:
            begin
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b0;
                AHB_HWDATA  <= 32'h0;
                i           <= i + 1'b1;
            end
            8'd3://write the yaddress = 0
            begin
                AHB_HADDR   <= 32'h00000004;
                AHB_HWRITE  <= 1'b1;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd4:
            begin
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b0;
                AHB_HWDATA  <= 32'h0;
                i           <= i + 1'b1;
            end
            8'd5://enable the erase function = 1
            begin
                AHB_HADDR   <= 32'h00000010;
                AHB_HWRITE  <= 1'b1;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd6:
            begin
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b0;
                AHB_HWDATA  <= 32'h2;
                i           <= i + 1'b1;
            end
            8'd7://enable the start
            begin
                AHB_HADDR   <= 32'h00000014;
                AHB_HWRITE  <= 1'b1;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd8:
            begin
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b0;
                AHB_HWDATA  <= 32'h1;
                i           <= i + 1'b1;
            end
            8'd9://read status
            begin
                AHB_HADDR   <= 32'h00000018;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd10:
            begin
                AHB_HWRITE  <= 1'b0;
                if( AHB_HRDATA == 32'h01)
                    i          <= i + 1'b1;
                else
                    i          <= i - 1'b1;
            end
            8'd11://clear the done status
            begin
                AHB_HADDR   <= 32'h00000018;
                AHB_HWRITE  <= 1'b1;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd12:
            begin
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b0;
                AHB_HWDATA  <= 32'h0;
                i          <= 8'd0;
            end
            8'd13://Write data to Flash step
            begin //xaddress = 0
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b1;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd14:
            begin
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b0;
                AHB_HWDATA  <= 32'h0;
                i           <= i + 1'b1;
            end
            8'd15://Write data to Flash step
            begin //yaddress = 0
                AHB_HADDR   <= 32'h00000004;
                AHB_HWRITE  <= 1'b1;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd16:
            begin
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b0;
                AHB_HWDATA  <= 32'h0;
                i          <= i + 1'b1;
            end
            8'd17://Write data to Flash step
            begin //wdata = 32'hF8F6F5F4 
                AHB_HADDR   <= 32'h0000000C;
                AHB_HWRITE  <= 1'b1;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd18:
            begin
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b0;
                AHB_HWDATA  <= 32'hF8F6F5F4;
                i          <= i + 1'b1;
            end
            8'd19://Write data to Flash step
            begin //set the write mode 
                AHB_HADDR   <= 32'h00000010;
                AHB_HWRITE  <= 1'b1;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd20:
            begin
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b0;
                AHB_HWDATA  <= 32'h1;
                i           <= i + 1'b1;
            end
            8'd21://Write data to Flash step
            begin //set the start mode 
                AHB_HADDR   <= 32'h00000014;
                AHB_HWRITE  <= 1'b1;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd22:
            begin
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b0;
                AHB_HWDATA  <= 32'h1;
                i          <= i + 1'b1;
            end
            8'd23://read status
            begin
                AHB_HADDR   <= 32'h00000018;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd24:
            begin
                AHB_HWRITE  <= 1'b0;
                if( AHB_HRDATA == 32'h01)
                    i          <= i + 1'b1;
                else
                    i          <= i - 1'b1;
            end
            8'd25://Write data to Flash step
            begin //Clear the Status
                AHB_HADDR   <= 32'h00000018;
                AHB_HWRITE  <= 1'b1;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd26:
            begin
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b0;
                AHB_HWDATA  <= 32'h0;
                i           <= 8'b0;
            end
            8'd27://read data to Flash step
            begin //xaddress = 0000
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b1;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd28:
            begin
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b0;
                AHB_HWDATA  <= 32'h0;
                i           <= i + 1'b1;
            end
           8'd29://read data to Flash step
            begin //xaddress = 0000
                AHB_HADDR   <= 32'h00000004;
                AHB_HWRITE  <= 1'b1;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd30:
            begin
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b0;
                AHB_HWDATA  <= 32'h0;
                i           <= i + 1'b1;
            end
            8'd31://read data to Flash step
            begin //read  = 1
                AHB_HADDR   <= 32'h00000010;
                AHB_HWRITE  <= 1'b1;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd32:
            begin
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b0;
                AHB_HWDATA  <= 32'h0;
                i           <= i + 1'b1;
            end
            8'd33://read data to Flash step
            begin //start   = 1
                AHB_HADDR   <= 32'h00000014;
                AHB_HWRITE  <= 1'b1;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd34:
            begin
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b0;
                AHB_HWDATA  <= 32'h1;
                i           <= i + 1'b1;
            end
            8'd35://read status
            begin
                AHB_HADDR   <= 32'h00000018;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd36:
            begin
                AHB_HWRITE  <= 1'b0;
                if( AHB_HRDATA == 32'h01)
                    i          <= i + 1'b1;
                else
                    i          <= i - 1'b1;
            end
           8'd37://read data to Flash step
            begin //clear status   = 1
                AHB_HADDR   <= 32'h00000018;
                AHB_HWRITE  <= 1'b1;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd38:
            begin
                AHB_HADDR   <= 32'h00000000;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b0;
                AHB_HWDATA  <= 32'h0;
                i           <= i + 1'b1;
            end
            8'd39://read data to Flash step
            begin //clear status   = 1
                AHB_HADDR   <= 32'h00000008;
                AHB_HWRITE  <= 1'b0;
                AHB_HSEL    <= 1'b1;
                i           <= i + 1'b1;
            end
            8'd40:
            begin
                AHB_HWDATA  <= 32'h0;
                i           <= 8'b0;
            end		
        endcase
    end
end

assign data_out = AHB_HRDATA;

assign AHB_HSIZE = 3'b000;

Gowin_Flash_Controller_Top u_Gowin_Flash_Controller_Top
(
    .AHB_HRDATA     (AHB_HRDATA),
    .AHB_HREADY     (AHB_HREADY),
    .AHB_HRESP      (AHB_HRESP ),
    .AHB_HTRANS     (AHB_HTRANS),
    .AHB_HSIZE      (AHB_HSIZE ),
    .AHB_HWRITE     (AHB_HWRITE),
    .AHB_HADDR      (AHB_HADDR ),
    .AHB_HWDATA     (AHB_HWDATA),
    .AHB_HSEL       (AHB_HSEL  ),
    .AHB_HCLK       (clk       ),
    .AHB_HRESETn    (rstn      )
);

endmodule