//------------------------------------------------------------------
//FLASH256K Controller APB Interface top module
//------------------------------------------------------------------

module Gowin_Flash_Controller_Template
(
    input   wire        clk,    //50MHz clock
    input   wire        rstn,   //reset
    input   wire        key_write,
    input   wire        key_read,
    input   wire        key_erase,
    output  wire [31:0] data_out
);

//get the falling edge of the key_write
reg fall_key_write1,fall_key_write2;
wire fall_key_write = ~fall_key_write1  & fall_key_write2;

always @(posedge clk or negedge rstn)
begin
    if(~rstn)
    begin
        fall_key_write1 <= 1'b0;
        fall_key_write2 <= 1'b0;
    end
    else
    begin
        fall_key_write1 <= key_write;
        fall_key_write2 <= fall_key_write1;
    end
end

//get the failing edge of the key_read
reg fall_key_read1,fall_key_read2;
wire fall_key_read = ~fall_key_read1 & fall_key_read2;

always @(posedge clk or negedge rstn)
begin
    if(~rstn)
    begin
        fall_key_read1 <= 1'b0;
        fall_key_read2 <= 1'b0;
    end
    else
    begin
        fall_key_read1 <= key_read;
        fall_key_read2 <= fall_key_read1;
    end
end

reg fall_key_erase1,fall_key_erase2;
wire fall_key_erase = ~fall_key_erase1 & fall_key_erase2;

always @(posedge clk or negedge rstn)
begin
    if(~rstn)
    begin
        fall_key_erase1 <= 1'b0;
        fall_key_erase2 <= 1'b0;
    end
    else
    begin
        fall_key_erase1 <= key_erase;
        fall_key_erase2 <= fall_key_erase1;
    end
end

wire  [31:0]	PRDATA;
wire			PREADY;
reg			    PSEL;
reg			    PENABLE;
reg			    PWRITE;
reg	  [31:0]	PADDR;
reg	  [31:0]    PWDATA;

reg   [31:0]    rdata;
always @(negedge clk or negedge rstn) begin
    if(~rstn)begin
        rdata <= 32'h0;
    end else begin
        rdata <= PRDATA;
    end
end

reg   [ 7:0] i;
always @(posedge clk or negedge rstn)
begin
    if(~rstn)
    begin
        i          <= 8'd0;
        PSEL       <= 1'b0;
        PWRITE     <= 1'b0;
        PENABLE    <= 1'b0;
        PWDATA     <= 32'b0;
        PADDR      <= 32'b0;
    end
    else
    begin
        case (i)
            8'd0:
            begin
                if(fall_key_write)
                begin
                    i <= 8'd19;
                end 
                else if(fall_key_read)
                begin
                    i <= 8'd40;
                end
                else if(fall_key_erase)
                begin
                    i <= 8'd1;
                end
            end
            //========Erase Flash========
            8'd1://write xaddress = 0
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b1;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                PADDR    <= 32'h00000000;
                i        <= i + 1'b1;
            end
            8'd2:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd3:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                i        <= i + 1'b1;
            end
            8'd4://write yaddress = 0
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b1;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                PADDR    <= 32'h00000004;
                i        <= i + 1'b1;
            end
            8'd5:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd6:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                i        <= i + 1'b1;
            end
            8'd7://set erase mode
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b1;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h2;
                PADDR    <= 32'h00000010;
                i        <= i + 1'b1;
            end
            8'd8:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd9:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                i        <= i + 1'b1;
            end
            8'd10://enable start
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b1;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h1;
                PADDR    <= 32'h00000014;
                i        <= i + 1'b1;
            end
            8'd11:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd12:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                i        <= i + 1'b1;
            end
            8'd13://read status
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PADDR    <= 32'h00000018;
                i        <= i + 1'b1;
            end
            8'd14:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd15:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                if(rdata == 32'h01)
                    i    <= i + 1'b1;
                else
                    i    <= 8'd13;
            end

            8'd16://clear the done status
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b1;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                PADDR    <= 32'h00000018;
                i        <= i + 1'b1;
            end
            8'd17:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd18:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                i        <= 8'd0;
            end

            //======Write data to Flash=======
            8'd19://write xaddress = 0
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b1;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                PADDR    <= 32'h00000000;
                i        <= i + 1'b1;
            end
            8'd20:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd21:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                i        <= i + 1'b1;
            end
            8'd22://write yaddress = 0
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b1;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                PADDR    <= 32'h00000004;
                i        <= i + 1'b1;
            end
            8'd23:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd24:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                i        <= i + 1'b1;
            end
            8'd25://wdata = 32'hF8F6F5F4
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b1;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'hF8F6F5F4;
                //PWDATA   <= 32'hA1B2C3D4;
                PADDR    <= 32'h0000000C;
                i        <= i + 1'b1;
            end
            8'd26:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd27:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                i        <= i + 1'b1;
            end
            8'd28://set write mode
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b1;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h1;
                PADDR    <= 32'h00000010;
                i        <= i + 1'b1;
            end
            8'd29:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd30:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                i        <= i + 1'b1;
            end
            8'd31://enable start
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b1;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h1;
                PADDR    <= 32'h00000014;
                i        <= i + 1'b1;
            end
            8'd32:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd33:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                i        <= i + 1'b1;
            end
            8'd34://read status
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PADDR    <= 32'h00000018;
                i        <= i + 1'b1;
            end
            8'd35:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd36:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                if(rdata == 32'h01)
                    i    <= i + 1'b1;
                else
                    i    <= 8'd34;
            end

            8'd37://clear the done status
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b1;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                PADDR    <= 32'h00000018;
                i        <= i + 1'b1;
            end
            8'd38:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd39:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                i        <= 8'd0;
            end

            //======Read data from Flash=======
            8'd40://write xaddress = 0
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b1;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                PADDR    <= 32'h00000000;
                i        <= i + 1'b1;
            end
            8'd41:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd42:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                i        <= i + 1'b1;
            end
            8'd43://write yaddress = 0
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b1;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                PADDR    <= 32'h00000004;
                i        <= i + 1'b1;
            end
            8'd44:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd45:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                i        <= i + 1'b1;
            end
            8'd46://set read mode
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b1;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                PADDR    <= 32'h00000010;
                i        <= i + 1'b1;
            end
            8'd47:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd48:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                i        <= i + 1'b1;
            end
            8'd49://enable start
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b1;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h1;
                PADDR    <= 32'h00000014;
                i        <= i + 1'b1;
            end
            8'd50:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd51:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                i        <= i + 1'b1;
            end
            8'd52://read status
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PADDR    <= 32'h00000018;
                i        <= i + 1'b1;
            end
            8'd53:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd54:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                if(rdata == 32'h01)
                    i    <= i + 1'b1;
                else
                    i    <= 8'd52;
            end

            8'd55://clear the done status
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b1;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                PADDR    <= 32'h00000018;
                i        <= i + 1'b1;
            end
            8'd56:
            begin
                PENABLE  <= 1'b1;
                i        <= i + 1'b1;
            end
            8'd57:
            begin
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PWDATA   <= 32'h0;
                i        <= i + 1'b1;
            end
            8'd58://read data
            begin
                PSEL     <= 1'b1;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                PADDR    <= 32'h00000008;
                i        <= i + 1'b1;
            end
            8'd59:
            begin
                PENABLE  <= 1'b1;
                i        <= 8'd0;
            end
            /*8'd60:
            begin
                //#1;
                PSEL     <= 1'b0;
                PWRITE   <= 1'b0;
                PENABLE  <= 1'b0;
                i        <= 8'd0;
            end*/
        endcase
    end
end

assign data_out = PRDATA;

Gowin_Flash_Controller_Top u_Gowin_Flash_Controller_Top
(
  .PRDATA     (PRDATA),
  .PREADY     (PREADY),

  .PCLK       (clk),
  .PSEL       (PSEL),
  .PENABLE    (PENABLE),
  .PWRITE     (PWRITE),
  .PADDR      (PADDR),
  .PWDATA     (PWDATA),
  .PRESETn    (rstn)
);
endmodule
