//-----------------------------------------------------------------
//FLASH256K Controller Wishbone Interface testbench.
//-----------------------------------------------------------------

`timescale 1ns/1ps

module tb;

reg clk;
reg nreset;

initial
begin 
  clk = 0; 
  nreset = 0;
  #100 nreset = 1'b1;
end

always
begin
  #10 clk = ~clk; 
end

//wb state machine
reg [4:0]  wb_adr_i_reg;
reg [31:0] wb_dat_i_reg;
wire[31:0] wb_dat_o_wire;
reg [31:0] wb_dat_o_reg;
reg 	   wb_we_i_reg;
reg 	   wb_stb_i_reg;
reg 	   wb_cyc_i_reg;
reg [7:0]  i;

//State machine
always @(posedge clk or negedge nreset)
begin
  if(~nreset)
  begin
    i <= 8'd0;
    wb_we_i_reg <= 1'b0;
    wb_stb_i_reg <= 1'b0;
    wb_cyc_i_reg <= 1'b0;
    wb_adr_i_reg <= 5'h0;
    wb_dat_i_reg <= 32'h0;
    wb_dat_o_reg <= 32'h0;
  end
  else
  begin
  //Erase The Flash
    case (i)
      8'd0:
      begin
        wb_stb_i_reg <= 1'b1;
        wb_cyc_i_reg <= 1'b1;
        wb_adr_i_reg <= 5'h18;
        wb_we_i_reg  <= 1'b1;
        wb_dat_i_reg <= 32'h03; //Start and Erase
        i <= i + 1'b1;
      end
      8'd1:
      begin
        wb_we_i_reg <= 1'b0;
        i <= i + 1'b1;
      end
      8'd2://Write to Flash ---write data address
      begin
        wb_adr_i_reg <= 5'h00;//Xaddress seclect
        wb_we_i_reg <= 1'b1;
        wb_dat_i_reg <= 32'h000000;//xaddress 
        i <= i + 1'b1;
      end
      8'd3:
      begin
        wb_we_i_reg <= 1'b0;
        i <= i + 1'b1;
      end
      8'd4://Write to Flash ---write data address
      begin
        wb_adr_i_reg <= 5'h04;//yaddress seclect
        wb_we_i_reg <= 1'b1;
        wb_dat_i_reg <= 32'h000000;//yaddress 
        i <= i + 1'b1;
      end
      8'd5:
      begin
        wb_we_i_reg <= 1'b0;
        i <= i + 1'b1;
      end	
      8'd6://Write to Flash ---write data address
      begin
        wb_adr_i_reg <= 5'h0C;//yaddress seclect
        wb_we_i_reg <= 1'b1;
        wb_dat_i_reg <= 32'hFFFFFFFF;//yaddress 
        i <= i + 1'b1;
      end
      8'd7:
      begin
        wb_we_i_reg <= 1'b0;
        i <= i + 1'b1;
      end	
      8'd8://Write to Flash ---write data address
      begin
        wb_adr_i_reg <= 5'h18;//seclect cmd
        wb_we_i_reg <= 1'b1;
        wb_dat_i_reg <= 32'h5;//start and write 
        i <= i + 1'b1;
      end
      8'd9:
      begin
        wb_we_i_reg <= 1'b0;
        i <= i + 1'b1;
      end	
    endcase
  end	
end

Gowin_Flash_Controller_Top u_Gowin_Flash_Controller_Top
(
  .wb_clk_i(clk),// master clock input
  .wb_rst_i(~nreset),// synchronous active high reset
  .wb_addr_i(wb_adr_i_reg),// lower address bits
  .wb_data_i(wb_dat_i_reg),// databus input
  .wb_data_o(wb_dat_o_wire),// databus output
  .wb_we_i(wb_we_i_reg), // write enable input
  .wb_stb_i(wb_stb_i_reg),// stobe/core select signal
  .wb_cyc_i(wb_cyc_i_reg),// valid bus cycle input
  .wb_ack_o()// bus cycle acknowledge output
);

initial 
begin
  #1800_0000_0;
  $finish;
end

// DUMPFSDB is defined when -verdi is used with svcompile
initial 
begin
  $fsdbDumpfile("Gowin_Flash_Controller.fsdb");
  $fsdbDumpvars();
end

endmodule
