//-----------------------------------------------------------------
//FLASH608K Controller AHB Interface testbench.
//-----------------------------------------------------------------

`timescale 1ns/1ps

module tb; 

reg AHB_HCLK;		//as system clock
reg AHB_HRESETn;	//as system reset

initial 
begin 
  AHB_HCLK    = 0; 
  AHB_HRESETn = 0;
  #100 AHB_HRESETn = 1'b1;
end

always 
begin
  #10 AHB_HCLK = ~AHB_HCLK; 
end

wire [31:0]	AHB_HRDATA;
wire		AHB_HREADY;
wire [ 1:0]	AHB_HRESP ;
wire [ 1:0] AHB_HTRANS;
wire [ 2:0]	AHB_HSIZE ;

reg 		AHB_HWRITE;
reg  [31:0]	AHB_HADDR ;
reg  [31:0] AHB_HWDATA;
reg    		AHB_HSEL  ;
reg  [7:0]  i;

assign AHB_HTRANS = 2'b10;

always @(posedge AHB_HCLK or negedge AHB_HRESETn)
begin
    if(~AHB_HRESETn)
    begin
      i             <= 8'd0;
      AHB_HADDR     <= 5'b0;
      AHB_HWDATA    <= 32'h0;
      AHB_HSEL      <= 1'b0;
      AHB_HWRITE    <= 1'b0;
    end
    else
    begin
      case (i)
        8'd0://write xaddress = 0
        begin
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b1;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd1:
        begin
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b0;
          AHB_HWDATA  <= 32'h0;
          i          <= i + 1'b1;
        end
        8'd2://write the yaddress = 0
        begin
          AHB_HADDR   <= 32'h00000004;
          AHB_HWRITE  <= 1'b1;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd3:
        begin
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b0;
          AHB_HWDATA  <= 32'h0;
          i           <= i + 1'b1;
        end
        8'd4://enable the erase function = 1
        begin
          AHB_HADDR   <= 32'h00000010;
          AHB_HWRITE  <= 1'b1;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd5:
        begin
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b0;
          AHB_HWDATA  <= 32'h2;
          i           <= i + 1'b1;
        end
        8'd6://enable the start
        begin
          AHB_HADDR   <= 32'h00000014;
          AHB_HWRITE  <= 1'b1;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd7:
        begin
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b0;
          AHB_HWDATA  <= 32'h1;
          i           <= i + 1'b1;
        end
        8'd8://read status
        begin
          AHB_HADDR   <= 32'h00000018;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd9:
        begin
          AHB_HWRITE  <= 1'b0;
          if( AHB_HRDATA == 32'h01)
            i          <= i + 1'b1;
          else
            i          <= i - 1'b1;
        end
        8'd10://clear the done status
        begin
          AHB_HADDR   <= 32'h00000018;
          AHB_HWRITE  <= 1'b1;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd11:
        begin
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b0;
          AHB_HWDATA  <= 32'h0;
          i           <= i + 1'b1;
        end
        8'd12://Write data to Flash step
        begin //xaddress = 0
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b1;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd13:
        begin
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b0;
          AHB_HWDATA  <= 32'h0;
          i           <= i + 1'b1;
        end
        8'd14://Write data to Flash step
        begin //yaddress = 0
          AHB_HADDR   <= 32'h00000004;
          AHB_HWRITE  <= 1'b1;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd15:
        begin
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b0;
          AHB_HWDATA  <= 32'h0;
          i          <= i + 1'b1;
        end
        8'd16://Write data to Flash step
        begin //wdata = 32'hF8F6F5F4 
          AHB_HADDR   <= 32'h0000000C;
          AHB_HWRITE  <= 1'b1;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd17:
        begin
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b0;
          AHB_HWDATA  <= 32'hAACCDDEE;
          i           <= i + 1'b1;
        end
        8'd18://Write data to Flash step
        begin //set the write mode 
          AHB_HADDR   <= 32'h00000010;
          AHB_HWRITE  <= 1'b1;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd19:
        begin
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b0;
          AHB_HWDATA  <= 32'h1;
          i           <= i + 1'b1;
        end
        8'd20://Write data to Flash step
        begin //set the start mode 
          AHB_HADDR   <= 32'h00000014;
          AHB_HWRITE  <= 1'b1;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd21:
        begin
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b0;
          AHB_HWDATA  <= 32'h1;
          i           <= i + 1'b1;
        end
        8'd22://read status
        begin
          AHB_HADDR   <= 32'h00000018;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd23:
        begin
          AHB_HWRITE  <= 1'b0;
          if( AHB_HRDATA == 32'h01)
            i          <= i + 1'b1;
          else
            i          <= i - 1'b1;
        end
        8'd24://Write data to Flash step
        begin //Clear the Status
          AHB_HADDR   <= 32'h00000018;
          AHB_HWRITE  <= 1'b1;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd25:
        begin
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b0;
          AHB_HWDATA  <= 32'h0;
          i           <= i + 1'b1;
        end
        8'd26://read data to Flash step
        begin //xaddress = 0000
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b1;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd27:
        begin
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b0;
          AHB_HWDATA  <= 32'h0;
          i           <= i + 1'b1;
        end
        8'd28://read data to Flash step
        begin //xaddress = 0000
          AHB_HADDR   <= 32'h00000004;
          AHB_HWRITE  <= 1'b1;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd29:
        begin
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b0;
          AHB_HWDATA  <= 32'h0;
          i           <= i + 1'b1;
        end
        8'd30://read data to Flash step
        begin //read  = 1
          AHB_HADDR   <= 32'h00000010;
          AHB_HWRITE  <= 1'b1;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd31:
        begin
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b0;
          AHB_HWDATA  <= 32'h0;
          i           <= i + 1'b1;
        end
        8'd32://read data to Flash step
        begin //start   = 1
          AHB_HADDR   <= 32'h00000014;
          AHB_HWRITE  <= 1'b1;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd33:
        begin
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b0;
          AHB_HWDATA  <= 32'h1;
          i           <= i + 1'b1;
        end
        8'd34://read status
        begin
          AHB_HADDR   <= 32'h00000018;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd35:
        begin
          AHB_HWRITE  <= 1'b0;
          if( AHB_HRDATA == 32'h01)
            i          <= i + 1'b1;
          else
            i          <= i - 1'b1;
        end
        8'd36://read data to Flash step
        begin //clear status   = 1
          AHB_HADDR   <= 32'h00000018;
          AHB_HWRITE  <= 1'b1;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd37:
        begin
          AHB_HADDR   <= 32'h00000000;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b0;
          AHB_HWDATA  <= 32'h0;
          i           <= i + 1'b1;
        end
        8'd38://read data to Flash step
        begin //clear status   = 1
          AHB_HADDR   <= 32'h00000008;
          AHB_HWRITE  <= 1'b0;
          AHB_HSEL    <= 1'b1;
          i           <= i + 1'b1;
        end
        8'd39:
        begin
          AHB_HWDATA  <= 32'h0;
          i           <= i + 1'b1;
        end				
        endcase
    end
end

Gowin_Flash_Controller_Top u_Gowin_Flash_Controller_Top
(
  .AHB_HRDATA     (AHB_HRDATA),
  .AHB_HREADY     (AHB_HREADY),
  .AHB_HRESP      (AHB_HRESP ),
  .AHB_HTRANS     (AHB_HTRANS),
  .AHB_HSIZE      (AHB_HSIZE ),
  .AHB_HWRITE     (AHB_HWRITE),
  .AHB_HADDR      (AHB_HADDR ),
  .AHB_HWDATA     (AHB_HWDATA),
  .AHB_HSEL       (AHB_HSEL  ),
  .AHB_HCLK       (AHB_HCLK  ),
  .AHB_HRESETn    (AHB_HRESETn)
);

initial 
begin
  #1800_0000_0;
  $finish;
end

// DUMPFSDB is defined when -verdi is used with svcompile
initial 
begin
  $fsdbDumpfile("Gowin_Flash_Controller.fsdb");
  $fsdbDumpvars();
end

endmodule
