//-----------------------------------------------------------------
//FLASH608K Controller APB Interface testbench.
//-----------------------------------------------------------------

`timescale 1ns/1ps

module tb; 

wire  [31:0]	PRDATA;
wire			PREADY;

reg			    PCLK;   //as system clock
reg			    PSEL;
reg			    PENABLE;
reg			    PWRITE;
reg	  [31:0]	PADDR;
reg	  [31:0]    PWDATA;
reg			    PRESETn; //as system reset

reg   [31:0]	rdata;

//clock generate
initial begin 
  PCLK = 0;
  forever
  #10 PCLK = ~PCLK;
end

//reset generate
initial begin
  PRESETn = 0;
  #100;
  PRESETn = 1;
end

//PRDATA delay one cycle
always @(posedge PCLK or negedge PRESETn) begin
    if(~PRESETn)begin
        rdata <= 32'h0;
    end else begin
        rdata <= PRDATA;
    end
end

reg  [7:0]  i;
always @(posedge PCLK or negedge PRESETn)
begin
    if(~PRESETn)begin
      i <= 8'd1;
      PSEL       <= 1'b0;
      PWRITE     <= 1'b0;
      PENABLE    <= 1'b0;
      PWDATA     <= 32'b0;
      PADDR      <= 32'b0;
    end
    else begin
      case (i)
        //========Erase Flash========
        8'd1://write xaddress = 0
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b1;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            PADDR    <= 32'h00000000;
            i        <= i + 1'b1;
        end
        8'd2:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd3:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            i        <= i + 1'b1;
        end
        8'd4://write yaddress = 0
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b1;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            PADDR    <= 32'h00000004;
            i        <= i + 1'b1;
        end
        8'd5:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd6:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            i        <= i + 1'b1;
        end
        8'd7://set erase mode
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b1;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h2;
            PADDR    <= 32'h00000010;
            i        <= i + 1'b1;
        end
        8'd8:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd9:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            i        <= i + 1'b1;
        end
        8'd10://enable start
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b1;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h1;
            PADDR    <= 32'h00000014;
            i        <= i + 1'b1;
        end
        8'd11:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd12:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            i        <= i + 1'b1;
        end
        8'd13://read status
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PADDR    <= 32'h00000018;
            i        <= i + 1'b1;
        end
        8'd14:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd15:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;

            if(rdata == 32'h01) begin
                i    <= i + 1'b1;
            end else begin
                i    <= 8'd13;
            end
        end

        8'd16://clear the done status
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b1;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            PADDR    <= 32'h00000018;
            i        <= i + 1'b1;
        end
        8'd17:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd18:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            i        <= i + 1'b1;
        end

        //======Write data to Flash=======
        8'd19://write xaddress = 0
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b1;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            PADDR    <= 32'h00000000;
            i        <= i + 1'b1;
        end
        8'd20:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd21:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            i        <= i + 1'b1;
        end
        8'd22://write yaddress = 0
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b1;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            PADDR    <= 32'h00000004;
            i        <= i + 1'b1;
        end
        8'd23:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd24:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            i        <= i + 1'b1;
        end
        8'd25://wdata = 32'hF8F6F5F4
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b1;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'hF8F6F5F4;
            PADDR    <= 32'h0000000C;
            i        <= i + 1'b1;
        end
        8'd26:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd27:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            i        <= i + 1'b1;
        end
        8'd28://set write mode
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b1;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h1;
            PADDR    <= 32'h00000010;
            i        <= i + 1'b1;
        end
        8'd29:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd30:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            i        <= i + 1'b1;
        end
        8'd31://enable start
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b1;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h1;
            PADDR    <= 32'h00000014;
            i        <= i + 1'b1;
        end
        8'd32:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd33:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            i        <= i + 1'b1;
        end
        8'd34://read status
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PADDR    <= 32'h00000018;
            i        <= i + 1'b1;
        end
        8'd35:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd36:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;

            if(rdata == 32'h01) begin
                i    <= i + 1'b1;
            end else begin
                i    <= 8'd34;
            end
        end

        8'd37://clear the done status
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b1;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            PADDR    <= 32'h00000018;
            i        <= i + 1'b1;
        end
        8'd38:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd39:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            i        <= i + 1'b1;
        end


        //======Read data from Flash=======
        8'd40://write xaddress = 0
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b1;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            PADDR    <= 32'h00000000;
            i        <= i + 1'b1;
        end
        8'd41:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd42:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            i        <= i + 1'b1;
        end
        8'd43://write yaddress = 0
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b1;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            PADDR    <= 32'h00000004;
            i        <= i + 1'b1;
        end
        8'd44:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd45:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            i        <= i + 1'b1;
        end
        8'd46://set read mode
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b1;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            PADDR    <= 32'h00000010;
            i        <= i + 1'b1;
        end
        8'd47:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd48:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            i        <= i + 1'b1;
        end
        8'd49://enable start
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b1;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h1;
            PADDR    <= 32'h00000014;
            i        <= i + 1'b1;
        end
        8'd50:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd51:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            i        <= i + 1'b1;
        end
        8'd52://read status
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PADDR    <= 32'h00000018;
            i        <= i + 1'b1;
        end
        8'd53:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd54:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;

            if(rdata == 32'h01) begin
                i    <= i + 1'b1;
            end else begin
                i    <= 8'd52;
            end
        end

        8'd55://clear the done status
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b1;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            PADDR    <= 32'h00000018;
            i        <= i + 1'b1;
        end
        8'd56:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd57:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PWDATA   <= 32'h0;
            i        <= i + 1'b1;
        end
        8'd58://read data
        begin
            #1;
            PSEL     <= 1'b1;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            PADDR    <= 32'h00000008;
            i        <= i + 1'b1;
        end
        8'd59:
        begin
            #1;
            PENABLE  <= 1'b1;
            i        <= i + 1'b1;
        end
        8'd60:
        begin
            #1;
            PSEL     <= 1'b0;
            PWRITE   <= 1'b0;
            PENABLE  <= 1'b0;
            i        <= 8'd1;
        end
        endcase
    end
end

Gowin_Flash_Controller_Top u_Gowin_Flash_Controller_Top
(
  .PRDATA     (PRDATA),
  .PREADY     (PREADY),

  .PCLK       (PCLK),
  .PSEL       (PSEL),
  .PENABLE    (PENABLE),
  .PWRITE     (PWRITE),
  .PADDR      (PADDR),
  .PWDATA     (PWDATA),
  .PRESETn    (PRESETn)
);

initial 
begin
  #1800_0000_0;
  $finish;
end

// DUMPFSDB is defined when -verdi is used with svcompile
initial 
begin
  $fsdbDumpfile("Gowin_Flash_Controller.fsdb");
  $fsdbDumpvars();
end

endmodule
