//-----------------------------------------------------------------
//FLASH608K Controller Wishbone Interface testbench.
//-----------------------------------------------------------------

`timescale 1ns / 1ns

module tb;

reg clk;
reg nreset;

reg [7:0]  i;
reg [4:0]  wb_addr;
reg [31:0] wb_data_i;
wire[31:0] wb_data_o;
reg        wr_en;

always @(posedge clk or negedge nreset)
begin
  if(~nreset)
  begin
    i <= 8'd0;
    wb_addr <= 5'b0;
    wb_data_i <= 32'h0;
    wr_en <= 1'b0;
  end
  else
  begin
    case (i)
      8'd0://Erase the 0 page step
      begin
        wb_addr <= 5'b0;//Select the xaddress
        wr_en <= 1'b1;//write to wishbone bus
        wb_data_i <= 32'b0;//write data to bus
        i <= i + 1'b1;
      end
      8'd1:
      begin
        wb_addr <= 5'h04;//Select the yaddress
        wr_en <= 1'b1;//write to wishbone bus
        wb_data_i <= 32'b0;
        i <= i + 1'b1;
      end
      8'd2:
      begin
        wb_addr <= 5'h10;//Select the cmd register
        wr_en <= 1'b1;//write to wishbone bus
        wb_data_i <= 32'h2; //Erase cmd to wishbone bus
        i <= i + 1'b1;
      end
      8'd3:
      begin
        wb_addr <= 5'h14;//Select the start cmd register
        wr_en <= 1'b1;//write to wishbone bus
        wb_data_i <= 32'h1; //Erase cmd to wishbone bus
        i <= i + 1'b1;
      end
      8'd4:
      begin
        wr_en <= 1'b0;
        i <= i + 1'b1;
      end
      8'd5://Read status
      begin
        wb_addr <= 5'h18;
        i <= i + 1'b1;
      end
      8'd6://Read status
      begin
        i <= i + 1'b1;
      end
      8'd7://Read status
      begin
        i <= i + 1'b1;
      end
      8'd8:
      begin
        if(wb_data_o == 32'h01) //Done signal
        begin
          i <= i + 1'b1;
        end
      end
      8'd9://Clear the Status
      begin
        wb_addr <= 5'h18;//Select the sr register
        wr_en <= 1'b1;//write to wishbone bus
        wb_data_i <= 32'b0;//write data to sr register
        i <= i + 1'b1;
      end
      8'd10:
      begin
        wr_en <= 1'b0;
        i <= i + 1'b1;
      end	
      8'd11://Write data to Flash step
      begin
        wb_addr <= 5'b0;//Select the xaddress
        wr_en <= 1'b1;//write to wishbone bus
        wb_data_i <= 32'b0;//write data to bus
        i <= i + 1'b1;
      end
      8'd12:
      begin
        wb_addr <= 5'h04;//Select the yaddress
        wr_en <= 1'b1;//write to wishbone bus
        wb_data_i <= 32'b0;
        i <= i + 1'b1;
      end
      8'd13:
      begin
        wb_addr <= 5'h0C;//Select the wdata register
        wr_en <= 1'b1;//write to wishbone bus
        wb_data_i <= 32'hF8F6F5F4;
        i <= i + 1'b1;
      end
      8'd14:
      begin
        wb_addr <= 5'h10;//Select the cmd register
        wr_en <= 1'b1;//write to wishbone bus
        wb_data_i <= 32'h1; //Write cmd to wishbone bus
        i <= i + 1'b1;
      end
      8'd15:
      begin
        wb_addr <= 5'h14;//Select the start cmd register
        wr_en <= 1'b1;//write to wishbone bus
        wb_data_i <= 32'h1; //Erase cmd to wishbone bus
        i <= i + 1'b1;
      end
      8'd16:
      begin
        wr_en <= 1'b0;
		wb_data_i <= 32'h0;
        i <= i + 1'b1;
      end
      8'd17:
      begin
        wb_addr <= 5'h18;
        i <= i + 1'b1;
      end
      8'd18:
      begin
        i <= i + 1'b1;
        end
      8'd19:
      begin
        if(wb_data_o == 32'h01) //Done signal
        begin
          i <= i + 1'b1;
        end
      end
      8'd20://Clear the Status
      begin
        wb_addr <= 5'h18;//Select the sr register
        wr_en <= 1'b1;//write to wishbone bus
        wb_data_i <= 32'b0;//write data to sr register
        i <= i + 1'b1;
      end
      8'd21:
      begin
        wr_en <= 1'b0;
        i <= i + 1'b1;
      end		
      8'd22://Read data from Flash step
      begin
        wb_addr <= 5'b0;//Select the xaddress
        wr_en <= 1'b1;//write to wishbone bus
        wb_data_i <= 32'b0;//write data to bus
        i <= i + 1'b1;
      end
      8'd23:
      begin
        wb_addr <= 5'h04;//Select the yaddress
        wr_en <= 1'b1;//write to wishbone bus
        wb_data_i <= 32'b0;
        i <= i + 1'b1;
      end
      8'd24:
      begin
        wb_addr <= 5'h10;//Select the cmd register
        wr_en <= 1'b1;//write to wishbone bus
        wb_data_i <= 32'h0; //Read cmd to wishbone bus
        i <= i + 1'b1;
      end
      8'd25:
      begin
        wb_addr <= 5'h14;//Select the start cmd register
        wr_en <= 1'b1;//write to wishbone bus
        wb_data_i <= 32'h1; //Start cmd to wishbone bus
        i <= i + 1'b1;
      end
      8'd26:
      begin
        wb_addr <= 5'h18;
        if(wb_data_o == 32'h01) //Done signal
        begin
          i <= i + 1'b1;
        end
      end
      8'd27://Clear the Status
      begin
        wb_addr <= 5'h18;//Select the sr register
        wr_en <= 1'b1;//write to wishbone bus
        wb_data_i <= 32'b0;//write data to sr register
        i <= i + 1'b1;
      end
      8'd28:
      begin
        wr_en <= 1'b0;
        wb_addr <= 5'h08;
        i <= i + 1'b1;
      end				
    endcase
  end
end

Gowin_Flash_Controller_Top u_Gowin_Flash_Controller_Top
(
  .wb_clk_i(clk),// master clock input
  .wb_rst_i(~nreset),// synchronous active high reset
  .wb_addr_i(wb_addr),// lower address bits
  .wb_data_i(wb_data_i),// databus input
  .wb_data_o(wb_data_o),// databus output
  .wb_we_i(wr_en), // write enable input
  .wb_stb_i(1'b1),// stobe/core select signal
  .wb_cyc_i(1'b1),// valid bus cycle input
  .wb_ack_o()// bus cycle acknowledge output            
);

initial
begin 
  clk = 0; 
  nreset = 0;
  #100 nreset = 1'b1;
end

always
begin
  #10 clk = ~clk; 
end

initial
begin
  #1200_0000_0 $finish;
end

initial
begin
  $fsdbDumpfile("Gowin_Flash_Controller.fsdb");
  $fsdbDumpvars;
end
   
endmodule
