//----------------------------------------------------------------------
//FLASH96KA Controller Register Interface top module
//----------------------------------------------------------------------

module Gowin_Flash_Controller_Template
(
    input   wire    clk,
    input   wire    nreset,
    input   wire    key, 
    output  wire    [31:0] rdata_o 
); 

reg key1_temp1,key1_temp2; 

always @(posedge clk or negedge nreset)
begin
    if(~nreset)
    begin
        key1_temp1 <= 1'b0;
        key1_temp2 <= 1'b0;
 
    end
    else
    begin
        key1_temp1 <= key;
        key1_temp2 <= key1_temp1;
 
    end
end
wire key_write = ~key1_temp1 & key1_temp2; 


reg [4:0] i;
//Test State machine
reg [31:0] wdata;
reg w_r_signal_i;
wire sleep_i;
reg [6:0] wxaddr;
reg [5:0] wyaddr;
reg Start_signal_i;
wire Done_signal_o;
wire [31:0] rdata;
reg  [31:0] rdata_reg;
reg  erase_reg;
assign rdata_o = rdata_reg;
assign sleep_i = 1'b0; 


always @(posedge clk or negedge nreset)
begin
	if(~nreset)
	begin
		i <= 5'd0;
		wdata <= 32'd0;
		w_r_signal_i <= 1'b0;
		wxaddr <= 7'd0;
		wyaddr <= 6'd0;
		Start_signal_i <= 1'b0;
	end
	else
	begin
		case (i)
		5'd0:	
			begin
                if(key_write)
                begin
                    Start_signal_i <= 1'b1;
                    erase_reg <= 1'b1;
                    i <= i + 1'b1;
                end
			end
		5'd1:
			begin
				Start_signal_i <= 1'b0;
				erase_reg <= 1'b0;
				i <= i + 1'b1;
			end
		5'd2:
			begin
				if(Done_signal_o)
				begin
					i <= i + 1'b1;
				end
			end
		5'd3:
			begin
				wdata <= 32'hFF88FFAA;//wdata + 1'b1;
				wxaddr <= 7'd0;//wxaddr + 1'b1;
				wyaddr <= 7'd0;//wyaddr + 1'b1;
				w_r_signal_i <= 1'b1; //Write To Flash
				Start_signal_i <= 1'b1;//Start_to Write
				i <= i + 1'b1;
			end
		5'd4:
			begin
			    w_r_signal_i <= 1'b0;
				Start_signal_i <= 1'b0;
				i <= i + 1'b1;
			end
			
		5'd5:
			begin
				if(Done_signal_o) 
				i <= 5'd6;
			end
		5'd6:
			begin
			
				wxaddr <= 7'd0;
				wyaddr <= 7'd0;
			
				i <= i + 1'b1;
			end
		5'd7:
			begin
				w_r_signal_i <= 1'b0;		//Read Signal
				Start_signal_i <= 1'b1;	
				i <= i + 1'b1;
			end
			
		5'd8:
			begin
			    w_r_signal_i <= 1'b0;
				Start_signal_i <= 1'b0;
				i <= i + 1'b1;
			end
			
		5'd9:
			begin
				if(Done_signal_o) 
				i <= i + 1'b1;
			end
			
		5'd10:
			begin
				rdata_reg<=rdata;
				i <=5'd0;
			end
        default:
			i <=5'd0;
		endcase
	end
end


Gowin_Flash_Controller_Top u_Gowin_Flash_Controller_Top 
(
	.clk_i(clk),
	.nrst_i(nreset), 
	.wdata_i(wdata),
	.wr_en_i(w_r_signal_i),
	.wxaddr_i(wxaddr[5:0]),
	.wyaddr_i(wyaddr),
	.done_flag_o(Done_signal_o),
	.start_flag_i(Start_signal_i),
	.rdata_o(rdata),
	.erase_en_i(erase_reg),
	.sleep_i(sleep_i)
);

endmodule
