//-----------------------------------------------------------------
//FLASH96KA Controller Register Interface testbench.
//-----------------------------------------------------------------

`timescale 1ns/1ns

module tb;

reg clk;
reg nreset;
reg [4:0] i;

//Test State machine
reg [31:0] wdata;
reg wr_en;
reg [6:0] wxaddr;
reg [5:0] wyaddr;
reg start_flag;
reg [31:0] rdata_reg;
reg erase_en;
reg led_reg;

wire done_flag;
wire sleep_i;
wire [31:0] rdata;
assign sleep_i = 1'b0;

always @(posedge clk or negedge nreset)
begin
  if(~nreset)
  begin
    i <= 5'd0;
    wdata <= 32'd0;
    wr_en <= 1'b0;
    wxaddr <= 7'd0;
    wyaddr <= 6'd0;
    start_flag <= 1'b0;
    led_reg <= 1'b0;
    erase_en <= 1'b0;
  end
  else
  begin
    case (i)
      5'd0:   
      begin
        start_flag <= 1'b1;
        erase_en <= 1'b1;
        i <= i + 1'b1;
      end
      5'd1:
      begin
        start_flag <= 1'b0;
        erase_en <= 1'b0;
        i <= i + 1'b1;
      end
      5'd2:
      begin
        if(done_flag)
        begin
          i <= i + 1'b1;
        end
      end
      5'd3:
      begin
        wdata <= 32'h0000f8f8;//wdata + 1'b1;
        wxaddr <= 7'd0;//wxaddr + 1'b1;
        wyaddr <= 7'd0;//wyaddr + 1'b1;
        wr_en <= 1'b1; //Write To Flash
        start_flag <= 1'b1;//Start_to Write
        led_reg <= 1'b0;
        i <= i + 1'b1;
      end
      5'd4:
      begin
        wr_en <= 1'b0;
        start_flag <= 1'b0;
        i <= i + 1'b1;
      end
      5'd5:
      begin
        if(done_flag)
        begin
          i <= i + 1'b1;
        end
      end
      5'd6:
      begin
        wxaddr <= 7'd0;
        wyaddr <= 7'd0;
        i <= i + 1'b1;
      end
      5'd7:
      begin
        wr_en <= 1'b0;           //Read Signal
        start_flag <= 1'b1; 
        i <= i + 1'b1;
      end           
      5'd8:
      begin
        wr_en <= 1'b0;
        start_flag <= 1'b0;
        i <= i + 1'b1;
      end  
      5'd9:
      begin
        if(done_flag)
        begin 
          i <= i + 1'b1;
        end
      end
      5'd10:
      begin
        rdata_reg<=rdata;
        i <= i + 1'b1;
      end
      5'd11:
      begin
        if(rdata_reg != wdata)
        begin
          led_reg <= 1'b0;
        end
        else
        begin
          led_reg <= 1'b1;
        end
      end
    endcase
  end
end

Gowin_Flash_Controller_Top u_Gowin_Flash_Controller_Top
(
  .clk_i(clk),
  .nrst_i(nreset), 
  .wdata_i(wdata),
  .wr_en_i(wr_en),
  .wxaddr_i(wxaddr),
  .wyaddr_i(wyaddr),
  .done_flag_o(done_flag),
  .start_flag_i(start_flag),
  .sleep_i(sleep_i),
  .rdata_o(rdata),
  .erase_en_i(erase_en)
);

initial 
begin 
  clk = 0; 
  nreset = 0;
  #100 nreset = 1'b1;
end

always 
begin
  #10 clk = ~clk; 
end

initial 
begin
  #1800_0000_0;
  $finish;
end

//DUMPFSDB is defined when -verdi is used with svcompile
initial 
begin
  $fsdbDumpfile("Gowin_Flash_Controller.fsdb");
  $fsdbDumpvars();
end

endmodule
