
//FLASH96K Controller Register Interface top module.

module Gowin_Flash_Controller_Template
(
    input   wire        clk,//50MHZ
    input   wire        rstn,
    output  wire [31:0] data_out,
    input   wire        key_write,
    input   wire        key_read     
);

    reg temp_key_write1,temp_key_write2;
    reg temp_key_read1 ,temp_key_read2 ;
    always @(posedge clk or negedge rstn)
    begin
        if(~rstn)
        begin
            temp_key_write1  <=  1'b0;
            temp_key_write2  <=  1'b0;

            temp_key_read1   <=  1'b0;
            temp_key_read2   <=  1'b0;
        end
        else
        begin
            temp_key_write1  <= key_write;
            temp_key_write2  <= temp_key_write1;

            temp_key_read1   <= key_read;
            temp_key_read2   <= temp_key_read1;
        end
    end
 
   
    wire    fall_edge_write = ~temp_key_write1 & temp_key_write2;
    wire    fall_edge_read  = ~temp_key_read1  & temp_key_read2;

    //test state machine
    reg [3:0] n_state;

    reg       read;
    reg       prog;
    reg       clear_page;
    reg       write_page;
    reg       erase;
    reg       pre_prog;

    reg [31:0]data_in;

    reg [ 8:0]row_address;
    reg [ 5:0]colu_address;
    reg [ 5:0]page_address;

    wire      done_sig;
    reg [6:0] write_num;

    always @(posedge clk or negedge rstn)
    begin
        if(~rstn)
        begin
            n_state         <=  4'b0;

            data_in         <=  32'hFAFBFC89;
            row_address     <=  9'b0;
            colu_address    <=  6'b0;
            page_address    <=  6'b0;
            write_num       <=  6'b0;
        end
        else
        begin
            case (n_state)
            4'b0:
                begin
                    if(fall_edge_write) 
                    begin
                         n_state  <=  4'd1;
                    end
                    else if(fall_edge_read)
                    begin
                         n_state  <=  4'd6;
                    end
                end
            4'd1:
                begin
                    clear_page  <= 1'b1;
                    if(done_sig)
                    begin
                        clear_page  <= 1'b0;
                        n_state     <= 4'd2;
                    end 
                end
            4'd2:
                begin                
                        if(done_sig)
                        begin
                            write_page   <= 1'b0;
                            n_state      <= 4'd3;
                        end
                        else
                        begin
                            write_page   <= 1'b1;
                        end
                end
            4'd3:
                begin
                    pre_prog <= 1'b1;
                    if(done_sig)
                    begin
                        n_state  <= 4'd4;
                        pre_prog <= 1'b0;
                    end
                end
            4'd4:
                begin
                    erase  <= 1'b1;
                    if(done_sig)
                    begin
                        n_state  <= 4'd5;
                        erase    <= 1'b0;
                    end
                end
            4'd5:
                begin
                    prog   <= 1'b1;
                    if(done_sig)
                    begin
                       n_state  <= 4'd0;
                       prog     <= 1'b0;
                    end
                end


            4'd6:
                begin
                    if(done_sig)
                    begin
                        read <= 1'b0;
                        n_state  <= 4'd0;
                    end
                    else
                    begin
                        read <= 1'b1;
                    end
                end
            endcase
        end
    end

Gowin_Flash_Controller_Top u_Gowin_Flash_Controller
(
     .clk_i                 (clk),//50MHZ
     .nrst_i                (rstn),
     .r_en_i                (read),
     .clear_page_enable_i   (clear_page),
     .prog_enable_i         (prog),
     .write_page_enable_i   (write_page),
     .erase_en_i            (erase),
     .pre_prog_i            (pre_prog),

     .wyaddr_i              (row_address),
     .wxaddr_i              (colu_address),
     .page_address          (page_address),
     .wdata_i               (data_in),
     .rdata_o               (data_out),
     .done_flag_o           (done_sig) 
);

endmodule
